from __future__ import division
import HTSeq
import argparse
import sys
# import re
import os
import numpy as np
# import csv
# import collections
# import itertools
# from Bio.Seq import Seq
# from Bio.SeqRecord import SeqRecord
# from Bio import SeqIO
# ambiguous_re = re.compile("ambiguous\[(.*)\]")
# sample_re = re.compile("(.*)_[ACGT]{6}_L002_R1_001")

LEFT="left"
RIGHT="right"
CLUSTER_GFF_TYPE="experimental_result_region"
GAP_GFF_TYPE="insertion_site"
SINGLETON_INSERT_GFF_TYPE="junction"
"""
python2.7 $SCRIPTS/find_insert_clusters.py $COLLAB/AlspaughLab/clipped_mapped/PE_H99align_VSL_093013.clips.sort.bam > $COLLAB/AlspaughLab/clipped_mapped/clusters_v1.txt
"""

def main():
    parser = argparse.ArgumentParser(description="Identifies read clusters (sets of overlapping reads), MORE")
    parser.add_argument("SAM_FILE", type=file, help="")
    parser.add_argument("--maxgap", type=int, metavar="GAP_LENGTH", default=100, 
                        help="Clusters separated by no more than %(metavar)s) bases are considered to be cluster doublets (default: %(default)s)")
    parser.add_argument("--minreads", type=int, metavar="NUM_READS", default=2, 
                        help="Clusters consisting of less than %(metavar)s reads are ignored (default: %(default)s)")
    parser.add_argument("--gff",type=argparse.FileType('w'),metavar="GFF_FILE",
                        help="Output results in GFF format to %(metavar)s")
    # parser.add_argument("--outdir", metavar="OUTDIR", help="Save output file to %(metavar)s (default is same directory as SAM_FILE)")
    # parser.add_argument("--verbose", help="increase output verbosity",action="store_true",default=False)
    args = parser.parse_args()

    
    # out_fastq_name = os.path.splitext(args.SAM_FILE.name)[0] + ".clips.fastq"
    # if args.outdir:
    #     out_fastq_name = os.path.join(args.outdir,os.path.basename(out_fastq_name))
    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50
    read_iter = load_sam_or_bam(args.SAM_FILE.name)
    cluster_list = find_clusters(read_iter)
    cluster_list = filter_clusters(cluster_list, args.minreads)
    
    for cluster in cluster_list:
        print "{0.count:<10} {0.range}".format(cluster)

    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50
    cluster_group_list = find_cluster_pairs(cluster_list,args.maxgap)
    # SeqIO.write(record_list, out_fastq_name, "fastq-sanger")
    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50
    for group in cluster_group_list:
        print group

    if args.gff:
        generate_gff(args.gff,cluster_group_list)
    # print >>sys.stderr, "="*60
    # for match in match_list:
    #     print match
    #     # print >>sys.stderr, "MATCH:", match.get_gff_line()
    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50


def generate_gff(gff_outhandle, cluster_group_list):
    print "NEED TO FIGURE OUT INSERT STRAND"
    for i, group in enumerate(cluster_group_list):
        for feature in group.get_features(i+1):
            gff_outhandle.write(feature.get_gff_line())
    """
            /Users/josh/Documents/BioinfCollabs/docs/sofa.obo
    name: region
name: read_pair
name: match_set
name: match_part
name: read
name: flanking_region
name: clip
name: tag
name: match
name: insertion_site
name: sequence_difference
name: junction
name: remark
name: experimental_result_region
name: gap
name: sequence_alteration
name: biomaterial_region
name: experimental_feature
name: paired_end_fragment
name: gene_group
name: adjacent_to
name: associated_with
name: connects_on
name: contained_by
name: contains
name: derives_from
name: disconnected_from
name: edited_from
name: edited_to
name: lost
name: overlaps
name: position_of
            """
def filter_clusters(cluster_list, minreads):
    return [cluster for cluster in cluster_list if cluster.count >= minreads]
    
def find_cluster_pairs(cluster_list,max_gap_size):
    cluster_list.sort()
    metacluster_list = []
    i = 0
    while i < len(cluster_list):
        cur_cluster = cluster_list[i]
        if i == len(cluster_list)-1:
            metacluster_list.append(ClusterSingleton(cur_cluster))
            print >>sys.stderr, "Last cluster is a singleton:", metacluster_list[-1]
            break
        else:
            next_cluster = cluster_list[i+1]

        print >>sys.stderr, "cur_cluster:{0}\tnext_cluster:{1}".format(cur_cluster, next_cluster)
        if (cur_cluster.iv.chrom != next_cluster.iv.chrom):
            print >>sys.stderr, "Last cluster on chrom:", cur_cluster
            metacluster_list.append(ClusterSingleton(cur_cluster))
            print >>sys.stderr, "Found a cluster singleton:", metacluster_list[-1]
            i += 1
        elif cur_cluster.insertion_side == RIGHT and next_cluster.insertion_side == LEFT:
            print >>sys.stderr, "Clusters face each other:", cur_cluster, next_cluster
            # for clust in (cur_cluster, next_cluster):
            #     print >>sys.stderr, clust
            #     for read in clust.read_list:
            #         print read
            metacluster = ClusterDoublet(cur_cluster,next_cluster)
            if metacluster.gap_length <= max_gap_size:
                metacluster_list.append(metacluster)
                print >>sys.stderr, "Found a cluster doublet:", metacluster.gap_length, metacluster
                i += 2
            else:
                print >>sys.stderr, "Gap is too big:", cur_cluster, next_cluster, metacluster.gap_length
                metacluster_list.append(ClusterSingleton(cur_cluster))
                print >>sys.stderr, "Found a cluster singleton:", metacluster_list[-1]
                i += 1
        else:
            metacluster_list.append(ClusterSingleton(cur_cluster))
            print >>sys.stderr, "Found a cluster singleton:", metacluster_list[-1]
            i += 1
    return metacluster_list
        
def load_sam_or_bam(sam_filename):
    sambase,samext = os.path.splitext(sam_filename)
    if samext == ".sam":
        align_seq = iter(HTSeq.SAM_Reader( sam_filename ))
    elif samext == ".bam":
        align_seq = iter(HTSeq.BAM_Reader( sam_filename ))
    else:
        print >>sys.stderr, "Problem with SAM/BAM File:", sam_filename
        sys.exit(1)
    return align_seq

# def find_clusters(align_seq):
#     cluster_list = []
#     cur_cluster = None
#     for aread in align_seq:
#         if not aread.aligned:
#             continue
#         elif cur_cluster == None:
#             #prime the pump
#             cur_cluster = ReadCluster(aread)
#             cluster_list.append(cur_cluster)
#         elif cur_cluster.overlaps(aread):
#             # aread overlaps cur_cluster, so it should be added
#             cur_cluster.add_read(aread)
#             # cur_cluster.add_read(aread)
#             # cur_cluster_iv.extend_to_include(aread.iv)
#         else:
#             # aread doesn't overlap cur_cluster, so time to start a new cluster
#             # print "Finished Cluster:", cur_cluster.range, cur_cluster.count
#             cur_cluster = ReadCluster(aread)
#             cluster_list.append(cur_cluster)
#     # cluster_list.append(cur_cluster)
#     # print cur_cluster.range, cur_cluster.count
#     return cluster_list

class ReadCluster:
    def __init__(self,aread):
        self.read_list = [aread]
        self.secondary_name = aread.secondary_frag.rname
        print >>sys.stderr, aread, "||",  aread.iv, "||", repr(aread.iv), "||", aread.readname
        self.iv = aread.iv.copy()
        # self.secondary_iv = aread.secondary_frag.iv.copy()
        self.secondary_iv = aread.secondary_frag.copy()
        print "-"*60, "\nINIT CHECK STRAND", aread.iv, aread.secondary_frag, aread.insert_side, aread.insert_point, aread.readname
        self.iv.strand = self.secondary_iv.strand = "."
        self._insert_point = aread.insert_point
        self._insert_side = aread.insert_side

    def add_read(self, aread):
        # print "self.iv", self.iv, type(self.iv)
        # print "aread", aread
        # print "aread.iv", aread.iv, type(aread.iv)
        print "ADD CHECK STRAND", aread.iv, aread.secondary_frag, aread.insert_side, aread.insert_point, aread.readname

        self.iv.extend_to_include(aread.iv)
        # print >>sys.stderr, self.secondary_iv
        self.secondary_iv.extend_to_include(aread.secondary_frag)
        self.read_list.append(aread)

    def matches_cluster(self,aread):
        return (self.overlaps(aread) and
                self.secondary_name == aread.secondary_frag.rname and
                self.insertion_side == aread.insert_side)

    def overlaps(self,aread):
        return self.iv.overlaps(aread.iv)

    @property
    def range(self):
        return "{0.chrom}:{0.start}-{0.end}".format(self.iv)
        # return self.iv

    @property
    def count(self):
        return len(self.read_list)

    @property
    def insertion_point(self):
        self._determine_insertion_point()
        return self._insert_point

    @property
    def insertion_side(self):
        # if self._insert_side == None:
        #     self._determine_insertion_point()
        return self._insert_side

    def _determine_insertion_point(self):
        insert_median = int(np.median([read.insert_point for read in self.read_list]))
        self._insert_point = insert_median
        # if len(self.read_list) == 1:
        #     self._insert_point = -1
        #     self._insert_side = "?"
        # start_median = int(np.median([read.iv.start for read in self.read_list]))
        # end_median = int(np.median([read.iv.end for read in self.read_list]))

        # frac_reads_start_median = sum(read.iv.start == start_median for read in self.read_list)/len(self.read_list)
        # frac_reads_end_median = sum(read.iv.end == end_median for read in self.read_list)/len(self.read_list)

        # if (frac_reads_start_median > 0.5) and (frac_reads_end_median > 0.5):
        #     print "Can't decide which end has insert point"
        # elif (frac_reads_start_median > 0.5):
        #     self._insert_point = start_median
        #     self._insert_side = LEFT
        # elif (frac_reads_end_median > 0.5):
        #     self._insert_point = end_median
        #     self._insert_side = RIGHT
        # else:
        #     self._insert_point = -2
        #     self._insert_side = "?!!"
        
        # # all_reads_start_median = all(read.iv.start == start_median for read in self.read_list)/len(self.read_list)
        # # all_reads_end_median = all(read.iv.end == end_median for read in self.read_list)/len(self.read_list)

        # # if all_reads_start_median and all_reads_end_median:
        # #     print "Can't decide which end has insert point"
        # # elif all_reads_start_median:
        # #     self._insert_point = start_median
        # #     self._insert_side = LEFT
        # # elif all_reads_end_median:
        # #     self._insert_point = end_median
        # #     self._insert_side = RIGHT
        # # else:
        # #     self._insert_point = -1
        # #     self._insert_side = "?"

    def __lt__(self,other):
        if self.iv.chrom == other.iv.chrom:
            return self.iv.start < other.iv.start
        else:
            return self.iv.chrom < other.iv.chrom

    # def distance(self,other):
    #     return max(self.iv.start-other.iv.end, other.iv.start-self.iv.end)

    def __str__(self):
        return "{0.iv.chrom}:{0.iv.start}-{0.iv.end} LENGTH:{0.iv.length} COUNT:{0.count} INSERT:{0.insertion_side} {0.insertion_point}".format(self)

    @property
    def insert_details(self):
        print self.secondary_iv, self.secondary_iv.start, self.secondary_iv.end, self.secondary_iv.start_d, self.secondary_iv.end_d
    # def gap(self,other):
    #     return (other.iv.start - self.iv.end)-1
    def __iter__(self):
        return tuple(self.read_list)

    def next(self):
        return self.read_list.next()

    def draw(self):
        if self.insertion_side == LEFT:
            sorted_reads = sorted(self.read_list,reverse=True,key=lambda read: (read.iv.end,read.iv.start,))
        else:
            sorted_reads = sorted(self.read_list,key=lambda read: (read.iv.start,read.iv.end))
        return [read.draw(i) for i,read in enumerate(sorted_reads)]

class ClusterGroup:
    Header = ("# type",
              "ref_chrom",
              "left_start_b1", "left_junction_b1", "left_numreads",
              "gap_length", "insert_length",
              "insert_chrom", "insert_start", "insert_end", "insert_strand",
              "right_junction_b1","right_end_b1", "right_numreads")
    def __init__(self):
        pass

class ClusterDoublet(ClusterGroup):
    Type = "pair"
    def __init__(self,left_cluster, right_cluster):
        self.left = left_cluster
        self.right = right_cluster
        self.iv = left_cluster.iv.copy()
        self.iv.extend_to_include(right_cluster.iv)

        self.insert_iv = left_cluster.secondary_iv.copy()
        print >>sys.stderr, "left_cluster:{0.secondary_iv}\tright_cluster:{1.secondary_iv}".format(left_cluster,right_cluster)
        self.insert_iv.extend_to_include(right_cluster.secondary_iv)
        if left_cluster.secondary_iv.start > right_cluster.secondary_iv.start:
            self.insert_iv.strand = "-"
        else:
            self.insert_iv.strand = "+"
        # self.gap = left_cluster.distance(right_cluster)

    def __str__(self):
        return "LEFT: {0.left}\nGAP: {0.gap_length} INSERT: {0.insert_length} {0.insert_iv} \nRIGHT: {0.right}\n\n".format(self)

    @property
    def str_with_secondary(self):
        return "LEFT: {0.left}, left_insert:{0.left.secondary_iv}\nGAP: {0.gap_length}\nRIGHT: {0.right}, right_insert: {0.right.secondary_iv}\n\n".format(self)

    @property
    def output(self):
        # ref_chrom, left_start_b1, left_junction_b1, gap_length, insert_length, insert_strand, insert_chrom, insert_start, insert_end, right_start, right_junction
        # print >>sys.stderr, "number of reads"; sys.exit(1)
        return (self.__class__.Type,
                self.left.iv.chrom,
                self.left.iv.start+1, self.left.iv.end,self.left.count,
                self.gap_length, self.insert_length,
                self.insert_iv.chrom, self.insert_iv.start+1, self.insert_iv.end,self.insert_iv.strand,
                self.right.iv.start+1, self.right.iv.end,self.right.count)

    @property
    def gap_length(self):
        return self.right.iv.start-self.left.iv.end

    @property
    def insert_length(self):
        return self.insert_iv.length
    
    def get_features(self,cluster_number):
        left_iv = self.left.iv.copy()
        left_iv.strand="+"
        left_feature = HTSeq.GenomicFeature("cluster_{0}_left".format(cluster_number), CLUSTER_GFF_TYPE, left_iv)
        left_feature.score = self.left.count
        
        right_iv = self.right.iv.copy()
        right_iv.strand="-"
        right_feature = HTSeq.GenomicFeature("cluster_{0}_right".format(cluster_number), CLUSTER_GFF_TYPE, right_iv)
        right_feature.score = self.right.count

        if self.gap == 0:
            gap_iv = HTSeq.GenomicInterval( left_iv.chrom, left_iv.end-1, right_iv.start+1, "." )
        else:
            # gap_iv = HTSeq.GenomicInterval( left_iv.chrom, left_iv.end+1, right_iv.start-1, "." )
            gap_iv = HTSeq.GenomicInterval( left_iv.chrom, left_iv.end, right_iv.start, "." )
        insert_name = "cluster_{0}_insert".format(cluster_number)
        insert_feature = HTSeq.GenomicFeature(insert_name, GAP_GFF_TYPE, gap_iv)
        # print dir(insert_feature)
        # insert_feature.__setattribute__("length",self.gap)
        insert_feature.attr = {"ID":insert_name, "length": self.gap}

        
        return (left_feature, insert_feature, right_feature)
        # attr:             The last (9th) column of a GFF file contains attributes, i.e. a list of name/value pairs. These are transformed into a dict, such that, e.g., gf.attr['gene_id'] gives the value of the attribute gene_id in the feature described by GenomicFeature object gf. The parser for the attribute field is reasonably flexible to deal with format variations (it was never clearly established whetehr name and value should be sperarated by a colon or an equal sign, and whether quotes need to be used) and also does a URL style decoding, as is often required.
    def draw(self):
        right_rect_list = self.right.draw()
        left_rect_list = self.left.draw()
        return right_rect_list+left_rect_list, max(len(right_rect_list),len(left_rect_list))
        
class ClusterSingleton(ClusterGroup):
    Type = "singleton"
    def __init__(self,singleton_cluster):
        self.singleton = singleton_cluster
        self.iv = singleton_cluster.iv

    def __str__(self):
        return "SINGLETON: {0.singleton}\n\n".format(self)

    @property
    def str_with_secondary(self):
        return "SINGLETON: {0.singleton}, insert:{0.singleton.secondary_iv}\n\n".format(self)

    def get_features(self,cluster_number):
        singleton_iv = self.singleton.iv.copy()
        insert_iv = None
        if self.singleton.insertion_side == RIGHT:
            singleton_iv.strand="+"
            # insert_iv = HTSeq.GenomicInterval(singleton_iv.chrom, singleton_iv.end+1, singleton_iv.end+1, "." )
            insert_iv = HTSeq.GenomicInterval(singleton_iv.chrom, singleton_iv.end, singleton_iv.end+1, "." )
        elif self.singleton.insertion_side == LEFT:
            singleton_iv.strand="-"
            # insert_iv =HTSeq.GenomicInterval(singleton_iv.chrom, singleton_iv.start-1, singleton_iv.start-1, "." )
            insert_iv =HTSeq.GenomicInterval(singleton_iv.chrom, singleton_iv.start-1, singleton_iv.start, "." )
        else:
            singleton_iv.strand="."

        singleton_feature = HTSeq.GenomicFeature("cluster_{0}_singleton".format(cluster_number), CLUSTER_GFF_TYPE, singleton_iv)
        singleton_feature.score = self.singleton.count
        feature_list = [singleton_feature] 

        if insert_iv != None:
            insert_feature = HTSeq.GenomicFeature("cluster_{0}_junction".format(cluster_number), SINGLETON_INSERT_GFF_TYPE, insert_iv)
            feature_list.append(insert_feature)
        
        return feature_list

    @property
    def output(self):
        if self.singleton.insertion_side == RIGHT:
            return (self.__class__.Type,
                    self.iv.chrom,
                    self.iv.start, self.iv.end,self.singleton.count,
                    None, None,
                    None,None,None,None,
                    None,None,None)
        elif self.singleton.insertion_side == LEFT:
            return (self.__class__.Type,
                    self.iv.chrom,
                    None,None,None,
                    None, None,
                    None,None,None,None,
                    self.iv.start, self.iv.end,self.singleton.count)
    def draw(self):
        rect_list = self.singleton.draw()
        return rect_list, len(rect_list)

if __name__ == "__main__":
   main()




#!/usr/bin/env python

import argparse
import os
import re
import subprocess
import tempfile, shutil, atexit
import sys
import signal
import fileinput
import extract_chimeras

def main():
    parser = argparse.ArgumentParser(description="Extract all reads in SAM_FILE that map to REF_NAME, or have pair that maps to it (including fusion matches)")
    parser.add_argument("REF_GENOME", help="FASTA file containing reference genome that reads will be mapped to.")
    parser.add_argument("INSERT_SEQ", help="FASTA file containing the sequence of the insertion DNA fragment.")
    parser.add_argument("ADAPTER_FILE", help="FASTA file containing sequences of adapters to be removed from reads.")
    parser.add_argument("FASTQ_FILE", nargs="+", help="One file for single-end, two for paired end.  Files may be gzipped.")
    parser.add_argument("-t", "--tmpdir", help="Directory for outputing intermediate files (default: %(default)s).")
    parser.add_argument("--threads", type=int, default=1, help="Number of parallel processed to run (default: %(default)s).")
    parser.add_argument("--minseed", type=int, default=10, help="Minimum seed length for bwa meme (default: %(default)s).")

    parser.add_argument("--outfile", metavar="TABLE_FILE", default=sys.stdout,
                        type=argparse.FileType('w'),
                        help="Output detailed table of insertions to %(metavar)s")
    parser.add_argument("--minreads", type=int, metavar="NUM_READS", default=2, 
                        help="Clusters consisting of less than %(metavar)s reads are ignored (default: %(default)s)")
    parser.add_argument("--maxgap", type=int, metavar="GAP_LENGTH", default=5000, 
                        help="Clusters separated by no more than %(metavar)s) bases are considered to be cluster doublets (default: %(default)s)")

    
    # parser.add_argument("--insert", type=file, help="The sequence of the inserted DNA fragment.  Used to determine how to organize junctions.")
    # parser.add_argument("REF_NAME", help="Extract all reads w")
    args = parser.parse_args()

    if len(args.FASTQ_FILE)>2:
        msg = "Maximum number of FASTQ files is 2 (for read pairs).  You supplied {0}".format(len(args.FASTQ_FILE))
        raise argparse.ArgumentTypeError(msg)

    if args.tmpdir:
        tmpdir = args.tmpdir
        if not os.path.exists(tmpdir):
            os.makedirs(tmpdir)
    else:
        tmpdir = tempfile.mkdtemp(suffix="aimhii")
        atexit.register(shutil.rmtree, tmpdir)

    concatentated_genome = os.path.join(tmpdir,"ref_and_insert.fa")
    concatenate_files(concatentated_genome, [args.REF_GENOME, args.INSERT_SEQ])
    
    ref_index = run_bwaindex(concatentated_genome)
    trimmed_fastqs = run_fastqmcf(args.ADAPTER_FILE, args.FASTQ_FILE,tmpdir)

    if len(args.FASTQ_FILE)==2:
        joined_filenames = run_fastqjoin(trimmed_fastqs,tmpdir)
        pair_bamname = run_bwamem(ref_index,joined_filenames[:2], os.path.join(tmpdir,"pair"),args.threads,args.minseed)
        joined_bamname = run_bwamem(ref_index,[joined_filenames[2]], os.path.join(tmpdir,"joined"),args.threads,args.minseed)
        merged_bamname = run_samtools_merge(os.path.join(tmpdir,"merged.bam"),pair_bamname,joined_bamname)
        final_bamname = merged_bamname
    elif len(args.FASTQ_FILE)==1:
        final_bamname = run_bwamem(ref_index,trimmed_fastqs, os.path.join(tmpdir,"single"),args.threads,args.minseed)

    run_samtools_index(final_bamname)

    extract_chimeras.run_analysis(final_bamname, args.minreads, args.maxgap, args.outfile, args.INSERT_SEQ, insertonly=True)


def run_fastqmcf(adapter_file, fastq_file_list, outdir):
    # fastq-mcf info/illumina_adapter1.fasta synthetic/syn_R1_001.fastq.gz synthetic/syn_R2_001.fastq.gz -o final_fastqs/syn_R1.tmp.gz -o final_fastqs/syn_R2.tmp.gz
    outfile_list = []
    for fastq_name in fastq_file_list:
        path_changed = os.path.join(outdir,(os.path.basename(fastq_name)))
        name_match = re.match("^(.+?)(\.fastq)?(\.gz)?$",path_changed,re.IGNORECASE)
        print name_match.groups()
        # outname = re.sub("(.*)(\.fastq)?(\.gz)?",r'\1.trim\2\3',path_changed, re.IGNORECASE)
        outname = "".join([name_match.group(1), ".trim"] + [group for group in name_match.groups()[1:] if group != None])
        outfile_list.append(outname)
    if len(fastq_file_list) == 1:
        cmd = ("fastq-mcf", adapter_file, fastq_file_list[0], "-o", outfile_list[0])
    elif len(fastq_file_list) == 2:
        cmd = ("fastq-mcf", adapter_file, fastq_file_list[0], fastq_file_list[1], "-o",outfile_list[0], "-o",outfile_list[1])
    else:
        msg = "Maximum number of FASTQ files is 2 (for read pairs)."
        raise argparse.ArgumentTypeError(msg)
    print >>sys.stderr, "\n\nTRIMMING READS"
    print cmd
    # See https://blog.nelhage.com/2010/02/a-very-subtle-bug/ for explanation of "preexec_fn" below
    subprocess.check_output(cmd, preexec_fn=lambda: signal.signal(signal.SIGPIPE, signal.SIG_DFL))
    return outfile_list

##--------------
## RUN fastq-join
##--------------
def run_fastqjoin(fastq_file_list, outdir):
    # fastq-join final_fastqs/syn_R1.trim.fastq.gz final_fastqs/syn_R2.trim.fastq.gz -o final_fastqs/syn.%.tmp.gz
    # mv final_fastqs/syn.un1.tmp.gz final_fastqs/syn.un1.fastq.gz
    # mv final_fastqs/syn.un2.tmp.gz final_fastqs/syn.un2.fastq.gz
    # mv final_fastqs/syn.join.tmp.gz final_fastqs/syn.join.fastq.gz
    outfile_template = os.path.join(outdir, "%.fastq.gz")
    cmd = ("fastq-join", fastq_file_list[0], fastq_file_list[1], "-o", outfile_template)
    print >>sys.stderr, "\n\nJOINGING READS"
    print cmd
    # See https://blog.nelhage.com/2010/02/a-very-subtle-bug/ for explanation of "preexec_fn" below
    subprocess.check_output(cmd, preexec_fn=lambda: signal.signal(signal.SIGPIPE, signal.SIG_DFL))
    return [outfile_template.replace("%",middle) for middle in ("un1", "un2", "join")]

def run_bwaindex(reference_genome):
    # bwa index -p bwa_bug_test/synthetic/syn_concat bwa_bug_test/synthetic/syn_concat.fa 
    index_prefix = os.path.splitext(reference_genome)[0]
    cmd = ("bwa", "index", "-p", index_prefix, reference_genome)
    print >>sys.stderr, "\n\nINDEXING GENOME"
    print cmd
    subprocess.check_output(cmd)
    return index_prefix

def run_bwamem(genome_index, fastq_list,outfile_root,numthreads=1,minseed=10):
    # bwa mem -t 12 -k 10 synthetic/syn_concat final_fastqs/syn.un1.fastq.gz final_fastqs/syn.un2.fastq.gz | samtools view -Sb - > bwa/syn.pair_TMP
    # bwa mem -t 12 -k 10 synthetic/syn_concat final_fastqs/syn.join.fastq.gz | samtools view -Sb - > bwa/syn.join_TMP

    bwa_mem_cmd = ["bwa", "mem", "-t", str(numthreads), "-k", str(minseed), genome_index] + fastq_list
    print >>sys.stderr, "\n\RUNNING BWA"
    print bwa_mem_cmd
    p_bwa_mem = subprocess.Popen(bwa_mem_cmd, stdout=subprocess.PIPE) #,stderr=errhandle)

    sam_view_cmd = ["samtools", "view", "-Su", "-"]
    p_sam_view = subprocess.Popen(sam_view_cmd, stdin=p_bwa_mem.stdout, stdout=subprocess.PIPE) #,stderr=errhandle)

    sam_sort_cmd = ["samtools", "sort", "-",outfile_root]
    p_sam_sort = subprocess.Popen(sam_sort_cmd, stdin=p_sam_view.stdout) #,stderr=errhandle)

    p_bwa_mem.stdout.close()
    p_sam_view.stdout.close()
    p_sam_sort.communicate()[0]
    return outfile_root+".bam"

##--------------------------------
## MERGE pair and joined BAM Files
##--------------------------------
# mkdir -p results
def run_samtools_merge(mergebam, inbam1, inbam2):
    # samtools merge bwa/syn.merge.bam bwa/syn.pair.bam bwa/syn.join.bam

    merge_cmd = ["samtools", "merge", "-f", mergebam, inbam1, inbam2]
    print >>sys.stderr, "\n\MERGING BAMS"
    print merge_cmd
    subprocess.check_output(merge_cmd)
    return mergebam

def run_samtools_index(inbam):
    # samtools merge bwa/syn.merge.bam bwa/syn.pair.bam bwa/syn.join.bam
    index_cmd = ["samtools", "index", inbam]
    print >>sys.stderr, "\n\INDEX BAM"
    print index_cmd
    subprocess.check_output(index_cmd)
    return inbam

##--------------------------------
## Concatenate Files
##--------------------------------
def concatenate_files(outfilename,filelist):
    with open(outfilename, 'w') as fout:
        for line in fileinput.input(filelist):
            fout.write(line)

# make -f bwamem_chimeras.mk all SYNTHETIC="TRUE"  --dry-run

##--------------
## RUN fastq-mcf
##--------------
# fastq-mcf info/illumina_adapter1.fasta synthetic/syn_R1_001.fastq.gz synthetic/syn_R2_001.fastq.gz -o final_fastqs/syn_R1.tmp.gz -o final_fastqs/syn_R2.tmp.gz
# mv final_fastqs/syn_R1.tmp.gz final_fastqs/syn_R1.trim.fastq.gz
# mv final_fastqs/syn_R2.tmp.gz final_fastqs/syn_R2.trim.fastq.gz
# mkdir -p final_fastqs

##--------------
## RUN fastq-join
##--------------
# fastq-join final_fastqs/syn_R1.trim.fastq.gz final_fastqs/syn_R2.trim.fastq.gz -o final_fastqs/syn.%.tmp.gz
# mv final_fastqs/syn.un1.tmp.gz final_fastqs/syn.un1.fastq.gz
# mv final_fastqs/syn.un2.tmp.gz final_fastqs/syn.un2.fastq.gz
# mv final_fastqs/syn.join.tmp.gz final_fastqs/syn.join.fastq.gz

##------------------------
## INDEX reference genome
##------------------------
# bwa index synthetic/syn_concat.fa -p synthetic/syn_concat

##--------------
## RUN bwa mem
##--------------
# mkdir -p bwa
# bwa mem -t 12 -k 10 synthetic/syn_concat final_fastqs/syn.join.fastq.gz | samtools view -Sb - > bwa/syn.join_TMP
# mv bwa/syn.join_TMP bwa/syn.join.unsrtbam

# bwa mem -t 12 -k 10 synthetic/syn_concat final_fastqs/syn.un1.fastq.gz final_fastqs/syn.un2.fastq.gz | samtools view -Sb - > bwa/syn.pair_TMP
# mv bwa/syn.pair_TMP bwa/syn.pair.unsrtbam
# mkdir -p bwa

##--------------
## SORT BAM Files
##--------------
# samtools sort bwa/syn.join.unsrtbam  bwa/syn.join_TMP
# mv bwa/syn.join_TMP.bam bwa/syn.join.bam
# mkdir -p bwa
# samtools sort bwa/syn.pair.unsrtbam  bwa/syn.pair_TMP
# mv bwa/syn.pair_TMP.bam bwa/syn.pair.bam

##--------------------------------
## MERGE pair and joined BAM Files
##--------------------------------
# samtools merge bwa/syn.merge.bam bwa/syn.pair.bam bwa/syn.join.bam
# mkdir -p results


##--------------------------------
## Run extract_chimeras.py
##--------------------------------
# python2.7 scripts/extract_chimeras.py bwa/syn.merge.bam --junction results/syn.merge.junc.tmp --fusionreads results/syn.merge.fusread.tmp --insert synthetic/syn_insert.fa
# mv results/syn.merge.junc.tmp results/syn.merge.junc
# mv results/syn.merge.fusread.tmp results/syn.merge.fusread




if __name__ == "__main__":
    main()



import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
# from matplotlib.backends.backend_pdf import PdfPages
from matplotlib.ticker import FuncFormatter, MaxNLocator

import os

READ_HEIGHT = 1
READ_HEIGHT_INCHES = 0.07
READ_PITCH = READ_HEIGHT * 1.3
MARGIN=1
def plot_clusters(metacluster_list,plotfilename):

    plotbase,plotext = os.path.splitext(plotfilename)
    for i, metacluster in enumerate(metacluster_list):
        print "drawing metacluster:", metacluster
        rect_list,stack_count = metacluster.draw()
        fig = plt.figure(figsize=(8, READ_HEIGHT_INCHES*stack_count+MARGIN))
        ax = plt.axes(frameon=False)
        ax.get_xaxis().tick_bottom() # Turn off ticks at top of plot
        ax.axes.get_yaxis().set_visible(False) # Hide y axis


        # ax = fig.add_subplot(111)
        for rect in rect_list:
            ax.add_patch(rect)
        ax.set_xlim(metacluster.iv.start,metacluster.iv.end)
        ax.set_ylim(1,stack_count*READ_PITCH)
        ax.set_xlabel(metacluster.iv.chrom)
        ax.xaxis.set_major_formatter(FuncFormatter(lambda x, pos: '%.0f'%x))
        xmin, xmax = ax.get_xaxis().get_view_interval()
        ymin, ymax = ax.get_yaxis().get_view_interval()
        ax.add_artist(Line2D((xmin, xmax), (ymin, ymin), color='black', linewidth=2))
        plt.tight_layout()
        plt.savefig("{0}_{1}{2}".format(plotbase,i,plotext))



#!/usr/bin/env python

import HTSeq
import argparse
import sys
import pysam
import re
import os
from Bio import SeqIO
from find_insert_clusters import ReadCluster, filter_clusters, find_cluster_pairs, RIGHT, LEFT, ClusterGroup
import csv
from operator import attrgetter

from plot_metaclusters import plot_clusters, READ_PITCH, READ_HEIGHT
import matplotlib

# cigar_re = re.compile("(d+)([mM])(d+)[F](d+)[mM]")
OPPOSITE_STRAND = {"+":"-", "-":"+"}
OPPOSITE_SIDE = {RIGHT:LEFT, LEFT:RIGHT}
MINUS = "-";PLUS = "+"

def current_work():
    todo_list = [
        "Need to handle indels better (max indel size?)",
        "Are all outputs base 1!!!!???",
        "Formalize metacluster output.  Include insert length, insert orientation, and gap length",
        "Use original read information to get insert strand (normalized to ref on + strand) see ChimeraJunction.__init__",
        "Fix false singletons (facing wrong direction, should be merged with adjacent cluster pair) :SINGLETON: ref0:9613-10000 LENGTH:387 COUNT:11 INSERT:left 9880, insert:insert0:[1808,2000)/. and SINGLETON: ref0:10004-10430 LENGTH:426 COUNT:16 INSERT:right 10247, insert:insert0:[0,302)/.",
        "harmonize iv with insertion_point (fix current difference in metacluster output)",
        "Handle meta clusters where constituent clusters overlap (small deletion)",
        "Handle SA CIGAR strings with multiple matches - look for Number of matches != 1"
        ]
    todo_list.insert(0, "="*80)
    todo_list.append("="*80)
    
    for todo in todo_list:
        print >> sys.stderr, "*", todo
        print "*", todo

def main():
    parser = argparse.ArgumentParser(description="Extract all reads in SAM_FILE that map to REF_NAME, or have pair that maps to it (including fusion matches)")
    parser.add_argument("SAM_FILE", type=file, help="BAM/SAM file containing mapped reads")
    parser.add_argument("--insert", type=file, help="The sequence of the inserted DNA fragment.  Used to determine how to organize junctions.")
    # parser.add_argument("REF_NAME", help="Extract all reads w")
    parser.add_argument("--junction", metavar="JUNCTION_FILE",
                        type=argparse.FileType('w'),
                        help="Output junction table to %(metavar)s")
    parser.add_argument("--fusionreads", metavar="READ_FILE",
                        type=argparse.FileType('w'),
                        help="Output reads containing junctions to %(metavar)s")
    parser.add_argument("--table", metavar="TABLE_FILE", default=sys.stdout,
                        type=argparse.FileType('w'),
                        help="Output detailed table of insertions to %(metavar)s")
    # parser.add_argument("--cluster", action="store_true", help="Cluster primary_fragments of junction reads", default=False)
    # parser.add_argument("--insertonly", action="store_true", help="Filter out junctions that don't include the insert", default=False)
    parser.add_argument("--alljunctions", action="store_true", help="Process all junctions (default: if --insert is provide, exclude reads that don't contain the insert", default=False)
    parser.add_argument("--minreads", type=int, metavar="NUM_READS", default=2, 
                        help="Clusters consisting of less than %(metavar)s reads are ignored (default: %(default)s)")
    parser.add_argument("--maxgap", type=int, metavar="GAP_LENGTH", default=5000, 
                        help="Clusters separated by no more than %(metavar)s) bases are considered to be cluster doublets (default: %(default)s)")
    parser.add_argument("--plot", metavar="PLOT_FILE",
                        help="Generate plots of metaclusters")

    # parser.add_argument("--outdir", metavar="OUTDIR", help="Save output file to %(metavar)s (default is same directory as SAM_FILE)")
    # parser.add_argument("--verbose", help="increase output verbosity",action="store_true",default=False)
    # parser.add_argument("--group", help="group reads and output groups to separate FASTQ files.",action="store_true",default=False)
    args = parser.parse_args()

    junction_list, metacluster_list = run_analysis(args.SAM_FILE.name, args.minreads,
                                                   args.maxgap, args.table,
                                                   args.insert, not(args.alljunctions))
    #--------------------------------------------------
    if args.junction:
        output_junctions(junction_list, args.junction)
        
    if args.fusionreads:
        output_fusionreads(junction_list, args.fusionreads)

    if args.plot:
        plot_clusters(metacluster_list,args.plot)
    #--------------------------------------------------
    
def output_junctions(junction_list,junction_handle):
    # count the number of hits at each junction
    junction_count_dict = {}
    for curjunc in junction_list:
        junction_count_dict.setdefault(curjunc.junction_tuple,[]).append(curjunc)
        # junction_count_dict[curjunc.junction_tuple] = 1+junction_count_dict.get(curjunc.junction_tuple,0)
    for key in sorted(junction_count_dict.keys()):
        l_chrom, l_junc,r_chrom, r_junc = key
        print >>junction_handle, "{4}\t{0}:{1}~{2}:{3}".format(l_chrom, l_junc,
                                                             r_chrom, r_junc,
                                                             len(junction_count_dict[key]))

def output_fusionreads(junction_list, fusion_handle):
    # for curjunc in sorted(junction_list,key=lambda x: x.junction_tuple):
    for curjunc in junction_list:
        print >>fusion_handle, "{0.readname}\t{0}".format(curjunc)

def run_analysis(mapread_bam, minreads, maxgap, outhandle, insert_seq=None, insertonly=False):
    if insert_seq:
        insert_rec = SeqIO.read(insert_seq,"fasta")
        ChimeraJunction.InsertSeqID = insert_rec.id

    junction_list = find_chimeric_reads(mapread_bam)
    if insertonly and insert_seq:
        junction_list = [junction for junction in junction_list if junction.contains_insert]
    junction_list.sort()
    # find_chimeric_reads_htseq(args.SAM_FILE.name)

    
    print "CLUSTERING"
    cluster_list = cluster_junctions(junction_list)
    cluster_list = filter_clusters(cluster_list, minreads)
    cluster_list.sort()

    for cluster in cluster_list:
        print cluster

    for cluster in cluster_list:
        print "{0.count:<10} {0.range}".format(cluster)

    for cluster in cluster_list:
        print "DETAILS", cluster.insert_details


    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50
    metacluster_list = find_cluster_pairs(cluster_list,maxgap)
    print "NEEED TO MAKE VALUES base 1!!!!", "#"*50
    for metacluster in metacluster_list:
        print metacluster.str_with_secondary

    outwriter = csv.writer(outhandle,dialect=csv.excel)
    # print >>sys.stderr, "metacluster_list", metacluster_list
    outwriter.writerow(ClusterGroup.Header)
    for metacluster in metacluster_list:
        # print metacluster.output
        outwriter.writerow(metacluster.output)

    return junction_list, metacluster_list

def find_chimeric_reads(sam_filename):
    junction_list = []
    samfile = pysam.Samfile(sam_filename)
    # refs = samfile.references
    # chroms_re = re.compile("("+'|'.join(samfile.references)+")-("+'|'.join(samfile.references)+")")
    
    # for aread in samfile.fetch():
    print >>sys.stderr, "aread.qname, samfile.getrname(aread.tid), aread.cigarstring, aread.aend, aread.alen, aread.pos, aread.qend, aread.qlen, aread.qstart, tags['SA']"    
    for aread in samfile:
        tags = dict(aread.tags)
        if "SA" in tags:
            sa_string = tags["SA"]
            sa_list = (sa_string[:-1] if sa_string.endswith(';') else sa_string).split(';')
            # print "-"*60, "\n<SA MATCH>", aread
            if not aread.is_paired:
                readnum = 0
            elif aread.is_read1:
                readnum = 1
            elif aread.is_read2:
                readnum = 2
            else:
                raise StandardError, "Problem with read number"
            if (aread.flag & 0x900) == 0: # this the "primary line", SO HERE
                line_type = 'prime' #primary
            elif (aread.flag & 0x800) == 0x800: 
                line_type = 'suppl' #supplementary
            elif (aread.flag & 0x100) == 0x100: 
                line_type = 'secon' #secondary
            else:
                line_type = '?'

            rname = samfile.getrname(aread.tid)
            if (aread.flag & 0x900) == 0 : #read is primary
                if aread.is_reverse:
                    primary_strand = MINUS
                else:
                    primary_strand = PLUS
                print "{0.qname:20} r{3} {4}\t{1}:{0.pos}-{0.aend}[{0.alen}]\tQ:{0.qstart}-{0.qend}[{0.qlen}]\t{5}{0.cigarstring:10}\tSA<{2}>".format(aread,rname, sa_list,readnum,line_type,primary_strand)
                read_parts = ReadFragment.listFromCIGAR(aread.cigarstring, aread.pos, rname, primary_strand)
                for supline in sa_list:
                    print supline
                    rname,pos,sup_strand,CIGAR,mapQ,NM = supline.split(",")
                    pos_b0 = int(pos)-1
                    # if CIGAR.count("M") != 1:
                    #     print "Number of matches != 1 ", CIGAR, "SKIPPING THIS ONE!!!", "LOOK INTO MERGING ACROSS DELETES?"
                    #     continue
                    read_parts.extend(ReadFragment.listFromCIGAR(CIGAR, pos_b0, rname, sup_strand))
                read_parts.sort(key=attrgetter('qstart'))
                for part in read_parts:
                    print part
                for i in range(len(read_parts)-1):
                    cur_junction = ChimeraJunction(read_parts[i],read_parts[i+1],aread.qname)
                    print cur_junction, cur_junction.readname
                    junction_list.append(cur_junction)
                print ""
            else:
                print "{0.qname:20} r{3} {4}\t{1}:{0.pos}-{0.aend}[{0.alen}]\tQ:{0.qstart}-{0.qend}[{0.qlen}]\t{0.cigarstring:10}".format(aread,rname, sa_list,readnum,line_type)
    return junction_list

def cluster_junctions(junction_list):
    cluster_list = []
    for cur_junction in junction_list:
        added_to_cluster = False
        for cur_cluster in cluster_list:
            if cur_cluster.matches_cluster(cur_junction):
                cur_cluster.add_read(cur_junction)
                added_to_cluster = True
        if not added_to_cluster:
            # aread doesn't overlap cur_cluster, so time to start a new cluster
            cluster_list.append(ReadCluster(cur_junction))
    return cluster_list

class ChimeraJunction:
    InsertSeqID = None
    def __init__(self,l_part,r_part,qname):
        self.readname = qname

        ##========================
        ## Select primary fragment
        ##========================
        if (((l_part.rname == ChimeraJunction.InsertSeqID) or
            (r_part.rname == ChimeraJunction.InsertSeqID)) and
            (l_part.rname != r_part.rname)):
            # Organize relative to reference chrom, not insert
            if l_part.rname == ChimeraJunction.InsertSeqID:
                self.primary_frag = r_part # ref_part = r_part
                self.secondary_frag = l_part # insert_part = l_part
            else:
                self.primary_frag = l_part # insert_part
                self.secondary_frag = r_part # ref_part
        elif l_part.rname < r_part.rname or (l_part.rname == r_part.rname and
                                             (l_part.start < r_part.start or
                                              (l_part.start == r_part.start and
                                               l_part.end < r_part.end))):
            self.primary_frag = l_part
            self.secondary_frag = r_part
        elif l_part.rname > r_part.rname or (l_part.rname == r_part.rname and
                                             (l_part.start > r_part.start or
                                             (l_part.start == r_part.start and
                                               l_part.end > r_part.end))):
            self.primary_frag = r_part
            self.secondary_frag = l_part
        else:
            print "STRANGE READ!!! NEED TO CHECK!!! l_part.rname == r_part.rname and l_part.start == r_part.start??"
            print "qname:{0}\tl_part:{1}\tr_part:{2}".format(qname,l_part,r_part)
            self.primary_frag = l_part
            self.secondary_frag = r_part

        ##========================
        ## Normalize Junctions:
        ## primary_frag should always be on plus strand, so invert junctions where primary_frag is minus strand
        ##========================
        if self.primary_frag.strand == PLUS and self.secondary_frag.strand == PLUS :
            l_part.distal = l_part.start+1
            l_part.junc = l_part.end
            r_part.junc = r_part.start+1
            r_part.distal = r_part.end
        elif self.primary_frag.strand == MINUS and self.secondary_frag.strand == MINUS :
            self.primary_frag.strand = self.secondary_frag.strand = PLUS
            l_part,r_part = r_part,l_part
            l_part.distal = l_part.start+1
            l_part.junc = l_part.end
            r_part.junc = r_part.start+1
            r_part.distal = r_part.end
        elif self.primary_frag.strand == PLUS and self.secondary_frag.strand == MINUS :
            if self.primary_frag == l_part:
                l_part.distal = l_part.start+1
                l_part.junc = l_part.end
                r_part.junc = r_part.end
                r_part.distal = r_part.start+1
            else: # self.primary_frag == r_part:
                l_part.distal = l_part.end
                l_part.junc = l_part.start+1
                r_part.junc = r_part.start+1
                r_part.distal = r_part.end
        elif self.primary_frag.strand == MINUS and self.secondary_frag.strand == PLUS :
            self.primary_frag.strand,self.secondary_frag.strand = PLUS,MINUS
            l_part,r_part = r_part,l_part
            if self.primary_frag == l_part:
                l_part.distal = l_part.start+1
                l_part.junc = l_part.end
                r_part.junc = r_part.end
                r_part.distal = r_part.start+1
            else: # self.primary_frag == r_part:
                l_part.distal = l_part.end
                l_part.junc = l_part.start+1
                r_part.junc = r_part.start+1
                r_part.distal = r_part.end
        else:
            raise StandardError, "Problem with strand combinations"

        self.l_frag = l_part
        self.r_frag = r_part
        if self.primary_frag == l_part:
            self.insert_side = RIGHT
        else:
            self.insert_side = LEFT
            
        self.insert_point = self.l_frag.junc

        print "l.distal:{0.distal} l.junc:{0.junc} r.junc:{1.junc} r.distal:{1.distal}".format(l_part,r_part)
        print "l.start:{0.start} l.end:{0.end} r.start:{1.start} r.end:{1.end}".format(l_part,r_part)

        self.iv = self.primary_frag
           
        print "junction: {0.primary_frag.rname}:{0.primary_frag.junc}<->{0.secondary_frag.rname}:{0.secondary_frag.junc} insert_side:{0.insert_side}".format(self)

    def __str__(self):
        return "{0.l_frag.chrom}:{0.l_frag.junc}~{0.r_frag.chrom}:{0.r_frag.junc} insert:{0.insert_side}".format(self)

    # def __hash__(self):
    #     # return hash((self.l_chrom, self.l_junc),(self.r_chrom, self.r_junc))
    #     return hash(((self.chrom1, self.junc1),(self.chrom2, self.junc2)))
    @property
    def junction_tuple(self):
        return (self.l_frag.chrom, self.l_frag.junc, self.r_frag.chrom, self.r_frag.junc)

    @property
    def contains_insert(self):
        return (ChimeraJunction.InsertSeqID == self.primary_frag.rname or
                ChimeraJunction.InsertSeqID == self.secondary_frag.rname)

    def __lt__(self,other):
        if self.primary_frag.rname < other.primary_frag.rname:
            return True
        elif self.primary_frag.rname > other.primary_frag.rname:
            return False
        elif self.secondary_frag.rname < other.secondary_frag.rname:
            return True
        elif self.secondary_frag.rname > other.secondary_frag.rname:
            return False
        #--------------------------------------------------
        elif self.primary_frag.junc < other.primary_frag.junc:
            return True
        elif self.primary_frag.junc > other.primary_frag.junc:
            return False
        elif self.secondary_frag.junc < other.secondary_frag.junc:
            return True
        elif self.secondary_frag.junc > other.secondary_frag.junc:
            return False
        #--------------------------------------------------
        elif self.primary_frag.distal < other.primary_frag.distal:
            return True
        elif self.primary_frag.distal > other.primary_frag.distal:
            return False
        elif self.secondary_frag.distal < other.secondary_frag.distal:
            return True
        elif self.secondary_frag.distal > other.secondary_frag.distal:
            return False
        else:
            return self.readname < other.readname
    def draw(self,i):
        xy = (self.primary_frag.start,i*READ_PITCH)
        width = self.primary_frag.length
        height = READ_HEIGHT
        print "I am pretending to draw myself xy:{0}, width:{1}, height:{2}".format(xy,width,height)
        if self.insert_side == RIGHT:
            rect_color = "green"
        else:
            rect_color = "purple"        
        rect = matplotlib.patches.Rectangle( xy, width=width, height=height,facecolor=rect_color,edgecolor="none",alpha=0.6)
        # scale_factor = 0.4
        # if abs(dx) < scaled_headlen:
        #     headlen = 0
        # else:
        #     headlen = scaled_headlen
        # exon_arrow = ax.arrow(x1,y,dx,0, shape='full', width=exon_dy, length_includes_head=True, head_width=scale_factor,head_length=headlen, transform=trans,fc="black",alpha=0.9)

        return rect

class ReadFragment(HTSeq.GenomicInterval):
    def __init__(self,qstart,qend,pos,aend,rname,strand):
        self.qstart = qstart
        self.qend = qend
        self.rname = rname
        super(ReadFragment,self).__init__(rname,pos,aend,strand)
        print "SUPER", super(ReadFragment,self).__str__()

    def __lt__(self,other):
        # this is important for ordering fragments in find_chimeric_reads()
        # return self.qstart < other.qstart
        raise NotImplementedError

    def __str__(self):
        return "Q:{0.qstart}-{0.qend} -> {0.rname}:{0.start}-{0.end}{0.strand} ({0.start_d}-{0.end_d})".format(self)

    def extend(self,qstart,qend,pos,aend,rname,strand):
        """Extend ReadFragment to include portion beyond a delete"""
        if self.rname != rname:
            raise StandardError, "Incompatible reference sequences: {0} != {1}".format(self.rname, rname)
        elif self.strand != strand:
            raise StandardError, "Incompatible strands: {0} != {1}".format(self.strand, strand)
        else:
            print "Before extension: {0}".format(self)
            self.qstart = min(self.qstart,qstart)
            self.qend = max(self.qend,qend)
            new_iv = HTSeq.GenomicInterval(rname,pos,aend,strand)
            print "new_iv:", new_iv
            self.extend_to_include(new_iv)
            print "After extension: {0}".format(self)
        # raise NotImplementedError

    @classmethod
    def listFromCIGAR(cls, cigarstring,position_b0, refname, strand):
        read_parts = []
        if strand == MINUS: # need to reverse the CIGAR
            print "Reversing CIGAR for minus strand read fragment"
            cigarstring = "".join(reversed(re.findall("\d+[MIDNSHP=X]", cigarstring)))

        op_type_list = []
        for op in HTSeq.parse_cigar(cigarstring, position_b0, refname, strand):
            print op, op.query_from, op.query_to, op.ref_iv
            if op.type == "M":
                if "M" in op_type_list:
                    if len(op_type_list) >=2 and op_type_list[-1] == "D" and op_type_list[-2] == "M":
                        print "extending (D):", op, op.query_from, op.query_to, op.ref_iv
                        read_parts[-1].extend(op.query_from, op.query_to,op.ref_iv.start,op.ref_iv.end,op.ref_iv.chrom,strand)
                    elif len(op_type_list) >=2 and op_type_list[-1] == "I" and op_type_list[-2] == "M":
                        print "extending (I):", op, op.query_from, op.query_to, op.ref_iv
                        read_parts[-1].extend(op.query_from, op.query_to,op.ref_iv.start,op.ref_iv.end,op.ref_iv.chrom,strand)
                    else:
                        print "CIGAR WARNING: Number of matches > 1", cigarstring
                else:
                    print "appending:", op, op.query_from, op.query_to, op.ref_iv
                    suppl_frag = cls(op.query_from, op.query_to,op.ref_iv.start,op.ref_iv.end,op.ref_iv.chrom,strand)
                    read_parts.append(suppl_frag)
            op_type_list.append(op.type)
        return read_parts

if __name__ == "__main__":
    current_work()
    main()
    current_work()

